@@ -1,14 +1,21 @@
-from bespin.errors import MissingOutput, BadOption
+from bespin.errors import MissingOutput, BadOption, BadStack, BadJson
+from bespin.errors import StackDoesntExist
+from bespin import helpers as hp
 
+from input_algorithms.spec_base import NotSpecified
 from input_algorithms.dictobj import dictobj
+import logging
 import json
 import six
 import os
 
+log = logging.getLogger("bespin.option_spec.stack_objs")
+
 class Stack(dictobj):
     fields = [
           "bespin", "name", "key_name", "environment", "stack_json", "params_json"
         , "vars", "stack_name", "env", "build_after", "ignore_deps", "artifacts"
+        , "skip_update_if_equivalent", "tags"
         ]
 
     def dependencies(self, stacks):
@@ -42,20 +49,17 @@ class Stack(dictobj):
         if missing:
             raise BadOption("Some environment variables aren't in the current environment", missing=missing)
 
-    def resolve_output(self, output_name):
-        outputs = self.outputs
-        if output_name not in self.outputs:
-            raise MissingOutput(wanted=output_name, available=outputs.keys())
-        return self.outputs[output_name]
-
     @property
-    def outputs(self):
-        if not hasattr(self, "_outputs"):
-            self._outputs = {}
-        return self._outputs
+    def cloudformation(self):
+        if not hasattr(self, "_cloudformation"):
+            self._cloudformation = self.bespin.credentials.cloudformation(self.stack_name, self.bespin.region)
+        return self._cloudformation
 
     @property
     def params_json_obj(self):
+        if self.params_json is NotSpecified:
+            return None
+
         with open(self.params_json) as fle:
             params = fle.read()
 
@@ -67,7 +71,17 @@ class Stack(dictobj):
                         value = value.resolve()
                     params = params.replace(key, value)
 
-        return json.loads(params)
+        try:
+            return json.loads(params)
+        except ValueError as error:
+            raise BadJson("Couldn't parse the parameters", filename=self.params_json, stack=self.key_name, error=error)
+
+    @hp.memoized_property
+    def stack_json_obj(self):
+        try:
+            return json.load(open(self.stack_json))
+        except ValueError as error:
+            raise BadJson("Couldn't parse the stack", filename=self.stack_json, stack=self.key_name, error=error)
 
     @property
     def artifact_vars(self):
@@ -76,6 +90,19 @@ class Stack(dictobj):
                 value = value.format(**dict(env.pair for env in self.env))
                 yield var, value
 
+    def create_or_update(self):
+        log.info("Creating or updating the stack (%s)", self.stack_name)
+        status = self.cloudformation.wait()
+
+        if not status.exists:
+            log.info("No existing stack, making one now")
+            self.cloudformation.create(self.stack_json_obj, self.params_json_obj, self.tags.as_dict() or None)
+        elif status.complete:
+            log.info("Found existing stack, doing an update")
+            self.cloudformation.update(self.stack_json_obj, self.params_json_obj)
+        else:
+            raise BadStack("Stack could not be updated", name=self.stack_name, status=status.name)
+
 class StaticVariable(dictobj):
     fields = ["value"]
 
@@ -83,10 +110,18 @@ class StaticVariable(dictobj):
         return self.value
 
 class DynamicVariable(dictobj):
-    fields = ["stack", "output"]
+    fields = ["stack", "output", ("credentials", None), ("region", None)]
 
     def resolve(self):
-        return self.stack.resolve_output(self.output)
+        if isinstance(self.stack, six.string_types):
+            outputs = self.credentials.cloudformation(self.stack, self.region).outputs
+        else:
+            outputs = self.stack.cloudformation.outputs
+
+        if self.output not in outputs:
+            raise MissingOutput(wanted=self.output, available=outputs.keys())
+
+        return outputs[self.output]
 
 class Environment(dictobj):
     """A single environment variable, and it's default or set value"""
@@ -102,3 +137,19 @@ class Environment(dictobj):
         else:
             return self.env_name, os.environ[self.env_name]
 
+class Skipper(dictobj):
+    fields = ["var1", "var2"]
+
+    def resolve(self):
+        try:
+            v1 = self.var1().resolve()
+        except StackDoesntExist:
+            return False
+
+        try:
+            v2 = self.var2().resolve()
+        except StackDoesntExist:
+            return False
+
+        return v1 and v2 and v1 == v2
+
